﻿using pcif.Properties;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Reflection;
using System.Reflection.Emit;
using System.Resources;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Windows.Forms.VisualStyles.VisualStyleElement;

namespace pcif
{
    public partial class Form1 : Form
    {
        /// <summary>
        /// タイマー
        /// </summary>
        private System.Timers.Timer timer;

        /// <summary>
        /// 改行
        /// </summary>
        private readonly string CRLF = Environment.NewLine;

        /// <summary>
        /// プログラムデータ
        /// </summary>
        private string programData;

        /// <summary>
        /// 受信モード
        /// </summary>
        private enum ModeReceive
        {
            SAVE,
            PRTON
        }
        private ModeReceive modeReceive;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public Form1()
        {
            InitializeComponent();

            btnClose.Text = Message.GetMessage("ButtonClose");
            btnSettings.Text = Message.GetMessage("ButtonSettings");

            timer = new System.Timers.Timer(10000); // 10秒でタイムアウト
            timer.Elapsed += TimerElapsed;

            bool flagChange = false;

            // 作業用フォルダが未設定の場合はドキュメント・フォルダとする
            if (Settings.Default.FilePathSave == "")
            {
                Settings.Default.FilePathSave = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
                flagChange = true;
            }

            // 作業用フォルダが未設定の場合はドキュメント・フォルダとする
            if (Settings.Default.FullPathLoad == "")
            {
                Settings.Default.FullPathLoad = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
                flagChange = true;
            }

            if (Settings.Default.TextBoxFont == null) {
                Settings.Default.TextBoxFont = new System.Drawing.Font(
                                                "Consolas", 
                                                12F, 
                                                System.Drawing.FontStyle.Regular, 
                                                System.Drawing.GraphicsUnit.Point, 
                                                ((byte)(0)));
                flagChange = true;
            }

            if (flagChange)
            {
                Settings.Default.Save();
            }

            GetSerialPorts();
        }

        /// <summary>
        /// フォームロード
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void Form1_Load(object sender, EventArgs e)
        {
            Version version = Assembly.GetExecutingAssembly().GetName().Version;
            this.Text = $"PC Interface for PB/FX series Ver.{version}(beta)";

            btnList730exe.Visible = Settings.Default.List730Enable;
            btnBas730exe.Visible = Settings.Default.Bas730Enable;

            textBox1.Font = Settings.Default.TextBoxFont;
        }

        /// <summary>
        /// シリアル受信
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void SerialPort1_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            if (modeReceive == ModeReceive.SAVE)
            {
                timer.Stop();
            }

            // シリアルポートから読込
            string receivedData = serialPort1.ReadExisting();
            programData += receivedData;

            UpdateTextBox(receivedData);

            if (modeReceive == ModeReceive.SAVE)
            {
                timer.Interval = 2000;  // 受信中は2秒でタイムアウト
                timer.Start();
            }
        }

        /// <summary>
        /// タイマー
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void TimerElapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            // タイマ停止
            timer.Stop();
            UpdateTextBox($"{CRLF}stop receiving{CRLF}");

            // シリアルポート・クローズ
            if (serialPort1.IsOpen)
            {
                UpdateTextBox($"{serialPort1.PortName} close{CRLF}");
                serialPort1.Close();
            }

            // 受信データ確認
            if (programData == "")
            {
                UpdateTextBox($"no data{CRLF}");
                return;
            }

            // ファイル出力
            string fileName = Settings.Default.FileNameSave;
            string fileNameList = fileName;
            if (Settings.Default.UniqueNameEnable)
            {
                fileNameList += $"{DateTime.Now:yyyyMMddHHmmss}.{Settings.Default.ExtList}";
            }
            else
            {
                fileNameList += $".{Settings.Default.ExtList}";
            }
            string fullPathList = System.IO.Path.Combine(Settings.Default.FilePathSave, fileNameList);

            try
            {
                using (StreamWriter writer = new StreamWriter(fullPathList))
                {
                    writer.WriteLine(programData);
                }
                UpdateTextBox($"make {fullPathList}{CRLF}");

                System.Threading.Thread.Sleep(500);     // 100ms待機
                ChangeBasicFile(fullPathList);
            }
            catch (Exception ex)
            {
                UpdateTextBox($"Error:{ex}{CRLF}");
            }

            string message = Message.GetMessage("SaveEnd");
            UpdateTextBox($"{message}{CRLF}");
        }

        /// <summary>
        /// SAVEボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void ButtonSave_Click(object sender, EventArgs e)
        {
            // 有効なポート番号が選択されていることの確認
            if (!CheckPortName())
            {
                return;
            }

            // ファイル名を入力するダイアログを表示
            using (FormSave form = new FormSave())
            {
                form.ShowDialog();
                if (!form.result)
                {
                    UpdateTextBox($"{form.message}{CRLF}");
                    return;
                }
            }

            // シリアルポート・オープン
            modeReceive = ModeReceive.SAVE;
            serialPort1.PortName = comboBox1.Text;
            if (!serialPort1.IsOpen)
            {
                serialPort1.Open();
                UpdateTextBox($"{serialPort1.PortName} open{CRLF}");
            }
            programData = "";

            UpdateTextBox($"start receiving{CRLF}");
            string message = Message.GetMessage("SaveStart");
            UpdateTextBox($"{message}{CRLF}");

            timer.Interval = 10000; // 初回は10秒でタイムアウト
            timer.Start();
        }

        /// <summary>
        /// シリアル出力
        /// </summary>
        /// <param name="data">送信データ</param>
        private void SendDataOverSerialPort(string data)
        {
            // シリアルポート・オープン
            serialPort1.PortName = comboBox1.Text;
            if (!serialPort1.IsOpen)
            {
                serialPort1.Open();
                UpdateTextBox($"{serialPort1.PortName} open{CRLF}");
            }

            // シリアル出力開始
            UpdateTextBox($"start sending{CRLF}");

            foreach (char ch in data)
            {
                serialPort1.Write(ch.ToString());
                //UpdateTextBox(ch.ToString());
                System.Threading.Thread.Sleep(1);   // 1ms待機
            }
            UpdateTextBox($"stop sending{CRLF}");

            // 500ms待機、２つ目以降のポートの場合、ウエイトがないとクローズでハングする
            System.Threading.Thread.Sleep(500);

            try
            {
                // シリアルポート・クローズ
                UpdateTextBox($"{serialPort1.PortName} close{CRLF}");
                serialPort1.Close();
            }
            catch (Exception ex)
            {
                UpdateTextBox($"Error:{ex}{CRLF}");
            }
        }

        /// <summary>
        /// LOADボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void ButtonLoad_Click(object sender, EventArgs e)
        {
            // 有効なポート番号が選択されていることの確認
            if (!CheckPortName())
            {
                return;
            }

            // ファイル名を入力するダイアログを表示
            using (FormLoad form = new FormLoad())
            {
                form.ShowDialog();
                if (!form.result)
                {
                    UpdateTextBox($"{form.message}{CRLF}");
                    return;
                }
            }
            string fullPathLoad = Settings.Default.FullPathLoad;
            UpdateTextBox($"LOAD {fullPathLoad}{CRLF}");

            string message = Message.GetMessage("LoadStart");
            UpdateTextBox($"{message}{CRLF}");

            try
            {
                // プログラムファイルへの変換
                string fullPathList = fullPathLoad;
                if (ChangeFileList(fullPathLoad))
                {
                    fullPathList = Path.ChangeExtension(fullPathLoad, Settings.Default.ExtList);
                }

                // ファイルからデータを読み込む
                string textData = File.ReadAllText(fullPathList);

                // 100ms待機
                System.Threading.Thread.Sleep(100);

                // シリアルポートにデータを送信
                SendDataOverSerialPort(textData);
            }
            catch (Exception ex)
            {
                UpdateTextBox($"Error:{ex}{CRLF}");
            }

            message = Message.GetMessage("LoadEnd");
            UpdateTextBox($"{message}{CRLF}");
        }

        /// <summary>
        /// 有効なシリアルポートの取得
        /// </summary>
        private void GetSerialPorts()
        {
            // 利用可能なシリアルポートを取得
            string[] ports = SerialPort.GetPortNames();

            // ポート名を表示
            comboBox1.Items.Clear();
            foreach (string port in ports)
            {
                comboBox1.Items.Add(port);
            }

            // 最初のポートを設定
            if (comboBox1.Items.Count == 0)
            {
                string message = Message.GetMessage("ErrorConnection");
                UpdateTextBox($"{message}{CRLF}");
            }
            else
            {
                comboBox1.SelectedIndex = 0; // 最初のアイテムを選択
                serialPort1.PortName = comboBox1.Text;
            }
        }

        /// <summary>
        /// テキストボックスへの文字追加
        /// </summary>
        /// <param name="text"></param>
        private void UpdateTextBox(string text)
        {
            if (textBox1.InvokeRequired)
            {
                textBox1.Invoke((MethodInvoker)(() => textBox1.AppendText(text)));
            }
            else
            {
                textBox1.AppendText(text);
            }
        }

        /// <summary>
        /// PRT ON ボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void BtnPrton_Click(object sender, EventArgs e)
        {
            // 有効なポート番号が選択されていることの確認
            if (!CheckPortName())
            {
                return;
            }

            string message = Message.GetMessage("GuidePrintOut");

            // シリアルポート・オープン
            modeReceive = ModeReceive.PRTON;
            serialPort1.PortName = comboBox1.Text;
            if (!serialPort1.IsOpen)
            {
                serialPort1.Open();
                UpdateTextBox($"{serialPort1.PortName} open{CRLF}");
            }
            programData = "";

            btnPrtoff.Enabled = Enabled;

            btnSave.Enabled = false;
            btnLoad.Enabled = false;
            btnPrton.Enabled = false;
            btnSettings.Enabled = false;

            UpdateTextBox($"start receiving{CRLF}");
            UpdateTextBox($"{message}{CRLF}");
        }

        /// <summary>
        /// PRT OFF ボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void btnPrtoff_Click(object sender, EventArgs e)
        {
            UpdateTextBox($"{CRLF}stop receiving{CRLF}");
            // シリアルポート・クローズ
            if (serialPort1.IsOpen)
            {
                UpdateTextBox($"{serialPort1.PortName} close{CRLF}");
                serialPort1.Close();
            }

            // SaveFileDialogのインスタンスを作成します
            using (SaveFileDialog saveFileDialog = new SaveFileDialog())
            {
                string title = Message.GetMessage("GuidePrintFile");
                string files = Message.GetMessage("PrintFiles");
                string allFiles = Message.GetMessage("AllFiles");

                saveFileDialog.Title = title;
                saveFileDialog.Filter = $"{files}(*.txt)|*.txt|{allFiles}(*.*)|*.*";

                if (saveFileDialog.ShowDialog() == DialogResult.OK)
                {
                    string fullPath = saveFileDialog.FileName;
                    using (StreamWriter writer = new StreamWriter(fullPath))
                    {
                        writer.WriteLine(programData);
                    }
                    UpdateTextBox($"make {fullPath}{CRLF}");
                }
            }

            btnPrtoff.Enabled = false;

            btnSave.Enabled = true;
            btnLoad.Enabled = true;
            btnPrton.Enabled = true;
            btnSettings.Enabled = true;
        }

        /// <summary>
        /// 設定ボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void ButtonSettings_Click(object sender, EventArgs e)
        {
            using (FormSettings form = new FormSettings())
            {
                if (form.ShowDialog() == DialogResult.OK)
                {
                    textBox1.Font = Settings.Default.TextBoxFont;
                }
            }
            btnList730exe.Visible = Settings.Default.List730Enable;
            btnBas730exe.Visible = Settings.Default.Bas730Enable;
        }

        /// <summary>
        /// 終了ボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void ButtonClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// プログラムファイルからBASICファイルへの変換
        /// </summary>
        /// <param name="fullPathList">プログラムファイルのフルパス名</param>
        private void ChangeBasicFile(string fullPathList)
        {
            // list730を使用しないときは実行しない
            if (!Settings.Default.List730Enable)
            {
                return;
            }

            // 作業ディレクトリの指定
            string workingDirectory = Path.GetDirectoryName(fullPathList);
            Directory.SetCurrentDirectory(workingDirectory);

            // パラメータ作成
            string fileNameList = Path.GetFileName(fullPathList);
            string fileNameBas = Path.ChangeExtension(fileNameList, Settings.Default.ExtBas);
            string fullPsthList730exe = Settings.Default.List730Path;
            string option = $"{Settings.Default.List730Option} {fileNameList}";

            try
            {
                // BASICファイルへ変換するプロセスの作成
                UpdateTextBox($"{fullPsthList730exe} {option} > {fileNameBas}{CRLF}");
                ProcessStartInfo psi = new ProcessStartInfo(fullPsthList730exe, option);

                // 標準出力をリダイレクト
                psi.UseShellExecute = false;
                psi.RedirectStandardOutput = true;

                // プロセス開始
                Process process = Process.Start(psi);

                // 標準出力を読み取る
                string output = process.StandardOutput.ReadToEnd();

                // プロセスが終了するのを待つ
                process.WaitForExit();

                // 終了コードを取得
                int exitCode = process.ExitCode;
                if (exitCode != 0)
                {
                    UpdateTextBox($"Error:{exitCode}{CRLF}");
                    return;
                }

                // 正常終了なら出力結果をファイル保存
                using (StreamWriter writer = new StreamWriter(fileNameBas))
                {
                    writer.WriteLine(output);
                }
                UpdateTextBox($"make {fileNameBas}{CRLF}");
            }
            catch (Exception ex)
            {
                UpdateTextBox($"Error:{ex}{CRLF}");
            }

        }

        /// <summary>
        /// BASICファイルからプログラムファイルへの変換
        /// </summary>
        /// <param name="fullPathBas">BASICファイルのフルパス名</param>
        private bool ChangeFileList(string fullPathBas)
        {
            // bas730を使用しないときは実行しない
            if (!Settings.Default.Bas730Enable)
            {
                return true;
            }

            // 拡張子がBASICファイルではない時は変換処理しない
            if (Path.GetExtension(fullPathBas) != "." + Settings.Default.ExtBas) {
                return true;
            }

            string fileNameBas = Path.GetFileName(fullPathBas);
            string filePathBas = Path.GetDirectoryName(fullPathBas);
            Directory.SetCurrentDirectory(filePathBas);

            string fileNameList = Path.ChangeExtension(fileNameBas, Settings.Default.ExtList);
            string fullPathBas730exe = Settings.Default.Bas730Path;
            string option = $"{Settings.Default.Bas730Option} {fileNameBas} {fileNameList}";

            try
            {
                UpdateTextBox($"{fullPathBas730exe} {option}{CRLF}");
                ProcessStartInfo psi = new ProcessStartInfo(fullPathBas730exe, option);

                // プロセス開始
                Process process = Process.Start(psi);

                // プロセスが終了するのを待つ
                process.WaitForExit();

                // 終了コードを取得
                int exitCode = process.ExitCode;
                if (exitCode != 0)
                {
                    UpdateTextBox($"Error:{exitCode.ToString()}{CRLF}");
                    return false;
                }
                UpdateTextBox($"make {fileNameList}{CRLF}");
            }
            catch (Exception ex)
            {
                UpdateTextBox($"Error:{ex}{CRLF}");
            }

            return true;
        }

        /// <summary>
        /// BASICファイル変換ボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void BtnList730exe_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog fileDialog = new OpenFileDialog())
            {
                string msgProgramFiles = Message.GetMessage("ProgramFiles");
                string ext = Settings.Default.ExtList;
                string msgAllFiles = Message.GetMessage("AllFiles");
                fileDialog.Title = Message.GetMessage("TitleSelectProgramFile");
                fileDialog.InitialDirectory = Settings.Default.FilePathSave;    // 初期フォルダ
                fileDialog.Filter = $"{msgProgramFiles}(*.{ext})|*.{ext}|{msgAllFiles}(*.*)|*.*";  // 選択できる拡張子
                fileDialog.FilterIndex = 1;             // 初期の拡張子

                if (fileDialog.ShowDialog() == DialogResult.OK)
                {
                    ChangeBasicFile(fileDialog.FileName);
                }
            }
        }

        /// <summary>
        /// プログラムファイル変換ボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void BtnBas730exe_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog fileDialog = new OpenFileDialog())
            {
                string msgBasicFile = Message.GetMessage("BasicFiles");
                string ext = Settings.Default.ExtBas;
                string msgAllFiles = Message.GetMessage("AllFiles");
                fileDialog.Title = Message.GetMessage("TitleSelectBasicFile");
                fileDialog.InitialDirectory = Settings.Default.FullPathLoad;    // 初期フォルダ
                fileDialog.Filter = $"{msgBasicFile}(*.{ext})|*.{ext}|{msgAllFiles}(*.*)|*.*";  // 選択できる拡張子
                fileDialog.FilterIndex = 1;             // 初期の拡張子

                if (fileDialog.ShowDialog() == DialogResult.OK)
                {
                    ChangeFileList(fileDialog.FileName);
                }
            }
        }

        /// <summary>
        /// 有効なポート番号が選択されていることの確認
        /// </summary>
        /// <returns>
        /// true : 有効
        /// false: 無効
        /// </returns>
        private bool CheckPortName()
        {
            if (comboBox1.Text == "")
            {
                string message = Message.GetMessage("GuideSerialPort");
                UpdateTextBox($"{message}{CRLF}");
                return false;
            }
            return true;
        }
    }
}
