﻿using pcif.Properties;
using System;
using System.CodeDom;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Reflection.Emit;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Net.WebRequestMethods;
using static System.Windows.Forms.VisualStyles.VisualStyleElement;

namespace pcif
{
    public partial class FormLoad : Form
    {
        public string fileName;
        public bool result = false;
        public string message = "";

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public FormLoad()
        {
            InitializeComponent();
            this.Text = "LOAD";
        }

        /// <summary>
        /// フォーム読込
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void FormLoad_Load(object sender, EventArgs e)
        {
            label1.Text = Message.GetMessage("GuideProgramFile");
            label2.Text = Message.GetMessage("LoadFile");
            label4.Text = Message.GetMessage("GuideLoadFile");

            textBox1.Text = Settings.Default.FullPathLoad;

            buttonCancel.Text = Message.GetMessage("ButtonCancel");
        }

        /// <summary>
        /// ファイル選択ボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void BtnSelectFile_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog fileDialog = new OpenFileDialog())
            {
                string title = Message.GetMessage("TitleSelectProgramFile");
                string ext = Settings.Default.ExtList;
                string files = Message.GetMessage("ProgramFiles");
                string allFiles = Message.GetMessage("AllFiles");

                if (Settings.Default.Bas730Enable)
                {
                    title = Message.GetMessage("TitleSelectBasicFile");
                    ext = Settings.Default.ExtBas;
                    files = Message.GetMessage("BasicFiles");
                }
                string fullPath = textBox1.Text;
                fileDialog.Title = $"{title}";         // ダイアログ名
                fileDialog.InitialDirectory = Path.GetDirectoryName(fullPath);  // 初期フォルダ
                fileDialog.FileName = Path.GetFileName(fullPath);   // 初期選択ファイル名
                fileDialog.Filter = $"{files}(*.{ext})|*.{ext}|{allFiles}(*.*)|*.*";  // 選択できる拡張子
                fileDialog.FilterIndex = 1;                         // 初期の拡張子

                if (fileDialog.ShowDialog() == DialogResult.OK)
                {
                    // ユーザーが選択したフォルダのパスを取得します。
                    textBox1.Text = fileDialog.FileName;
                }
            }
        }

        /// <summary>
        /// OKボタン押下
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void ButtonOk_Click(object sender, EventArgs e)
        {
            bool changed = false;

            if (Settings.Default.FullPathLoad != textBox1.Text)
            {
                Settings.Default.FullPathLoad = textBox1.Text;
                changed = true;
            }

            if (changed)
            {
                Settings.Default.Save();
            }

            result = true;
            this.Close();
        }

        /// <summary>
        /// キャンセルボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            result = false;
            message = Message.GetMessage("ButtonCancel");
            this.Close();
        }
    }
}
