﻿using pcif.Properties;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Windows.Forms.VisualStyles.VisualStyleElement;

namespace pcif
{
    public partial class FormSave : Form
    {
        public string fileName;
        public bool result = false;
        public string message = "";

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="title">タイトル</param>
        /// <param name="fileName">ファイル名の初期値</param>
        public FormSave()
        {
            InitializeComponent();
            this.Text = "SAVE";
        }

        /// <summary>
        /// フォーム読込
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void FormSetFileName_Load(object sender, EventArgs e)
        {
            label1.Text = Message.GetMessage("GuideSave");
            label2.Text = Message.GetMessage("SaveFileName");
            label3.Text = Message.GetMessage("WorkFolder");
            label4.Text = Message.GetMessage("NotesSaveFileName");

            checkBox1.Text = Message.GetMessage("CheckUniqueFileName");
            checkBox1.Checked = Settings.Default.UniqueNameEnable;

            textBoxFileName.Text = Settings.Default.FileNameSave;
            textBoxFilePath.Text = Settings.Default.FilePathSave;

            buttonCancel.Text = Message.GetMessage("ButtonCancel");
        }

        /// <summary>
        /// 重複しないファイル名設定の変更
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void CheckBox1_CheckedChanged(object sender, EventArgs e)
        {
            Settings.Default.UniqueNameEnable = checkBox1.Checked;
        }

        /// <summary>
        /// フォルダ参照ボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void BtnSelectDirectory_Click(object sender, EventArgs e)
        {
            using (FolderBrowserDialog folderDialog = new FolderBrowserDialog())
            {
                folderDialog.SelectedPath = textBoxFilePath.Text;
                if (folderDialog.ShowDialog() == DialogResult.OK)
                {
                    // ユーザーが選択したフォルダのパスを取得します。
                    textBoxFilePath.Text = folderDialog.SelectedPath;
                }
            }
        }

        /// <summary>
        /// OKボタン押下
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void BtnOk_Click(object sender, EventArgs e)
        {
            if (textBoxFileName.Text == "")
            {
                message = Message.GetMessage("ErrorNoFileName");
                result = false;
                this.Close();
            }

            bool changed = false;

            if (Settings.Default.FileNameSave != textBoxFileName.Text)
            {
                Settings.Default.FileNameSave = textBoxFileName.Text;
                changed = true;
            }
            if (Settings.Default.FilePathSave != textBoxFilePath.Text)
            {
                Settings.Default.FilePathSave = textBoxFilePath.Text;
                changed = true;
            }
            if (Settings.Default.UniqueNameEnable != checkBox1.Checked)
            {
                Settings.Default.UniqueNameEnable = checkBox1.Checked;
            }

            if (changed)
            {
                Settings.Default.Save();
            }

            result = true;
            this.Close();
        }

        /// <summary>
        /// キャンセルボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void BtnCancel_Click(object sender, EventArgs e)
        {
            result = false;
            message = Message.GetMessage("ButtonCancel");
            this.Close();
        }
    }
}
