﻿using pcif.Properties;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Windows.Forms.VisualStyles.VisualStyleElement;

namespace pcif
{
    public partial class FormSettings : Form
    {
        /// <summary>
        /// テキストボックスのフォント
        /// </summary>
        private Font FontTextBox;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public FormSettings()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォームロード
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void FormSettings_Load(object sender, EventArgs e)
        {
            label1.Text = "";
            label3.Text = Message.GetMessage("ExtBasicFile");
            label4.Text = Message.GetMessage("ExtProgramFile");
            label5.Text = label7.Text = Message.GetMessage("FilePath");
            label6.Text = label8.Text = Message.GetMessage("Option");
            labelFont.Text = Settings.Default.TextBoxFont.Name;

            checkBox1.Text = Message.GetMessage("UseBas730");
            checkBox2.Text = Message.GetMessage("UseList730");

            textBox2.Text = Settings.Default.ExtBas;
            textBox3.Text = Settings.Default.ExtList;
            textBox4.Text = Settings.Default.Bas730Path;
            textBox5.Text = Settings.Default.Bas730Option;
            textBox6.Text = Settings.Default.List730Path;
            textBox7.Text = Settings.Default.List730Option;

            checkBox1.Checked = Settings.Default.Bas730Enable;
            checkBox2.Checked = Settings.Default.List730Enable;

            groupBox1.Enabled = checkBox1.Checked;
            groupBox2.Enabled = checkBox2.Checked;

            btnFont.Text = Message.GetMessage("ButtonFont");
            btnCancel.Text = Message.GetMessage("ButtonCancel");
        }

        /// <summary>
        /// bas730.exeファイル選択ボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void button4_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog fileDialog = new OpenFileDialog())
            {
                string exeFiles = Message.GetMessage("ExeFiles");
                string allFiles = Message.GetMessage("AllFiles");
                fileDialog.Title = "bas730.exe";       // ダイアログ名
                fileDialog.InitialDirectory = System.IO.Path.GetDirectoryName(textBox4.Text);   // 初期フォルダ
                fileDialog.FileName = "bas730.exe";     // 初期選択ファイル名
                fileDialog.Filter = "{exeFiles}(*.exe)|*.exe|{allFiles}(*.*)|*.*";  // 選択できる拡張子
                fileDialog.FilterIndex = 1;             // 初期の拡張子

                if (fileDialog.ShowDialog() == DialogResult.OK)
                {
                    // ユーザーが選択したフォルダのパスを取得します。
                    textBox4.Text = fileDialog.FileName;
                }
            }
        }

        /// <summary>
        /// list730.exeファイル選択ボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void button5_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog fileDialog = new OpenFileDialog())
            {
                string exeFiles = Message.GetMessage("ExeFiles");
                string allFiles = Message.GetMessage("AllFiles");
                fileDialog.Title = "list730.exe"; // ダイアログ名
                fileDialog.InitialDirectory = System.IO.Path.GetDirectoryName(textBox6.Text);   // 初期フォルダ
                fileDialog.FileName = "list730.exe";    // 初期選択ファイル名
                fileDialog.Filter = "{exeFiles}(*.exe)|*.exe|{allFiles}(*.*)|*.*";  // 選択できる拡張子
                fileDialog.FilterIndex = 1;             // 初期の拡張子

                if (fileDialog.ShowDialog() == DialogResult.OK)
                {
                    textBox6.Text = fileDialog.FileName;
                }
            }
        }

        /// <summary>
        /// bas730.exe有効
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            groupBox1.Enabled = checkBox1.Checked;
        }

        /// <summary>
        /// list730.exe有効
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {
            groupBox2.Enabled = checkBox2.Checked;
        }

        /// <summary>
        /// OKボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void BtnOk_Click(object sender, EventArgs e)
        {
            bool changed = false;

            if (Settings.Default.ExtBas != textBox2.Text)
            {
                Settings.Default.ExtBas = textBox2.Text;
                changed = true;
            }
            if (Settings.Default.ExtList != textBox3.Text)
            {
                Settings.Default.ExtList = textBox3.Text;
                changed = true;
            }
            if (Settings.Default.Bas730Path != textBox4.Text)
            {
                Settings.Default.Bas730Path = textBox4.Text;
                changed = true;
            }
            if (Settings.Default.Bas730Option != textBox5.Text)
            {
                Settings.Default.Bas730Option = textBox5.Text;
                changed = true;
            }
            if (Settings.Default.List730Path != textBox6.Text)
            {
                Settings.Default.List730Path = textBox6.Text;
                changed = true;
            }
            if (Settings.Default.List730Option != textBox7.Text)
            {
                Settings.Default.List730Option = textBox7.Text;
                changed = true;
            }
            if (Settings.Default.Bas730Enable != checkBox1.Checked)
            {
                Settings.Default.Bas730Enable = checkBox1.Checked;
                changed = true;
            }
            if (Settings.Default.List730Enable != checkBox2.Checked)
            {
                Settings.Default.List730Enable = checkBox2.Checked;
                changed = true;
            }
            if (FontTextBox != null)
            {
                Settings.Default.TextBoxFont = FontTextBox;
                changed = true;
            }

            if (changed)
            {
                Settings.Default.Save();
            }

            this.Close();
        }

        /// <summary>
        /// キャンセルボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void BtnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// フォント選択ボタン
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void BtnFont_Click(object sender, EventArgs e)
        {
            fontDialog1.Font = Settings.Default.TextBoxFont;
            if (fontDialog1.ShowDialog() == DialogResult.OK)
            {
                FontTextBox = fontDialog1.Font;
                labelFont.Text = FontTextBox.Name;
            }
        }
    }
}
